const webpack = require('webpack');
const path = require('path');
const glob = require('glob');
const merge = require('webpack-merge');
const fs = require('fs-extra');

// Plugins
const MiniCssExtractPlugin = require('mini-css-extract-plugin');
const CleanWebpackPlugin = require('clean-webpack-plugin');
const CopyPlugin = require('copy-webpack-plugin');
const TerserPlugin = require('terser-webpack-plugin');

const tempBuildPath = './sources/tmp';
const assetsPath = './assets';

var entries = {
  bootstrapper: glob.sync('./sources/js/parts/*.js'),
};

module.exports = {
  entry: () => scanFiles('./sources/js/*.js', './sources/sass/bootstrapper/*.scss'),

  output: {
    filename: 'js/[name].js',
    path: path.resolve(__dirname, '../sources/tmp')
  },

  module: {
    rules: [
      {
        test: /\.js$/,
        exclude: /(node_modules|bower_components)/,
        use: {
          loader: "babel-loader",
          options: {
            presets: ['@babel/preset-env']
          }
        }
      },
      {
        test: /\.(sa|sc|c)ss$/,
        use: [
          MiniCssExtractPlugin.loader,
          {
            loader: "css-loader",
            options: {
              sourceMap: true,
            }
          },
          {
            loader: "postcss-loader",
            options: {
              sourceMap: true,
              ident: 'postcss',
              plugins: [
                require('autoprefixer'),
              ]
            }
          },
          {
            loader: "sass-loader",
            options: {
              sourceMap: true,
            }
          }
        ]
      },
      {
        test: /\.(png|jpg|jpeg|gif|ico|svg)$/,
        use: [{
          loader: 'file-loader',
          options: {
            name: '[name].[ext]',
            outputPath: 'images',
          }
        }]
      },
      {
        test: /\.(woff|woff2|eot|ttf|otf)$/,
        use: [
          {
            loader: 'file-loader',
            options: {
              name: '[name].[ext]',
              outputPath: 'fonts',
            },
          }
        ]
      },
    ]
  },
  optimization: {
    minimizer: [
      new TerserPlugin({
        cache: true,
        parallel: true,
      }),
    ],
  },
  plugins: [
    new webpack.ProgressPlugin({ profile: false }),
    new CleanWebpackPlugin(tempBuildPath, {} ),
    new MiniCssExtractPlugin({
      filename: 'css/[name].css',
    }),
    new CopyPlugin([
      { from: 'sources/vendor', to: path.resolve(__dirname, '../sources/tmp/vendor') },
      { from: 'node_modules/bootstrap', to: path.resolve(__dirname, '../sources/tmp/vendor/bootstrap/') },
      { from: 'node_modules/popper.js', to: path.resolve(__dirname, '../sources/tmp/vendor') + '/popper.js/' },
      { from: 'sources/fonts', to: path.resolve(__dirname, '../sources/tmp/fonts/') },
      { from: 'sources/images', to: path.resolve(__dirname, '../sources/tmp/images/') },
    ]),
    // Check if build has done without compilation errors and synchronize assets from temporary directory.
    {
      apply: (compiler) => {
        compiler.hooks.done.tap('AssetsSync', stats => {
          if (stats.hasErrors() === false) {
            // Checking files existence in temporary directory
            fs.exists(tempBuildPath, function(exists) {
              if (exists) {
                // Clear assets directory
                fs.removeSync(assetsPath);
                // Copy new build from temporary directory to assets
                fs.copy(tempBuildPath, assetsPath, function (err) {
                  if (err) {
                    console.error(err);
                  } else {
                    console.log('Assets successfully sync!');
                  }
                });
              }
            });
          }
        });
      },
    },
  ],
};

function scanFiles(jsSources, scssSources) {
  if (jsSources.length !== 0 && scssSources.length !== 0) {
    let jsFiles = glob.sync(jsSources);
    let scssFiles = glob.sync(scssSources);

    if (jsFiles.length !== 0 || scssFiles.length !== 0) {
      let sourceFiles = merge(performEntries(jsFiles), performEntries(scssFiles));

      return merge(entries, sourceFiles);
    } else {
      return entries;
    }
  }
}

function performEntries(files) {
  if (files.length !== 0) {
    let sourcesObject = {};
    files.forEach(function (filename) {
      let propertyName = path.parse(filename.replace('.es6', '_es6')).name;
      sourcesObject[`${propertyName}`] = `${filename}`;
    });

    return sourcesObject;
  }
}
